@@ -5,8 +5,8 @@
   
   DESCRIPTION:
   
-  $Id: regression-storage.c,v 1.3 2003/10/21 18:28:05 ettore Exp $
-  $Locker:  $
+  $Id: regression-storage.c 393 2003-10-21 18:28:34Z ettore $
+  $Locker$
 
   (C) COPYRIGHT 1999 Eric Busboom 
   http://www.softwarestudio.org
