@@ -6,7 +6,7 @@
  *  Copyright 2001 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,7 +14,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -23,15 +23,16 @@
 
 #include <config.h>
 
+#include <string.h>
+
 #include "camel-exception.h"
-#include "camel-vtrash-folder.h"
+#include "camel-i18n.h"
+#include "camel-mime-message.h"
+#include "camel-private.h"
 #include "camel-store.h"
 #include "camel-vee-store.h"
-#include "camel-mime-message.h"
-#include "camel-i18n.h"
-#include "camel-private.h"
+#include "camel-vtrash-folder.h"
 
-#include <string.h>
 
 /* Returns the class for a CamelFolder */
 #define CF_CLASS(so) ((CamelFolderClass *)((CamelObject *)(so))->klass)
@@ -541,7 +542,7 @@ vtrash_remove_folder(CamelVeeFolder *vf, CamelFolder *sub)
 	for (i=0;i<infos->len;i++) {
 		CamelVeeMessageInfo *mi = infos->pdata[i];
 
-		if (mi == NULL)
+		if (mi == NULL || mi->real == NULL)
 			continue;
 
 		if (mi->real->summary == ssummary) {
