@@ -5,6 +5,7 @@
  * Authors:
  *  Sivaiah Nallagatla <snallagatla@novell.com>
  *  parthasarathi susarla <sparthasarathi@novell.com>
+ *  Sankar P <psankar@novell.com>
  *   
  *
  * Copyright (C) 2004, Novell Inc.
@@ -593,8 +594,8 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	CamelGroupwiseStorePrivate *priv = gw_store->priv;
 	CamelMessageInfo *info = NULL;
 	CamelGroupwiseMessageInfo *gw_info;
-	GList *read_items = NULL;
-	flags_diff_t diff;
+	GList *read_items = NULL, *deleted_read_items = NULL, *unread_items = NULL;
+	flags_diff_t diff, unset_flags;
 	const char *container_id;
 	EGwConnectionStatus status;
 	EGwConnection *cnc;
@@ -651,6 +652,8 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 		if (gw_info && (gw_info->info.flags & CAMEL_MESSAGE_FOLDER_FLAGGED)) {
 			do_flags_diff (&diff, gw_info->server_flags, gw_info->info.flags);
+			do_flags_diff (&unset_flags, flags, gw_info->server_flags);
+			
 			diff.changed &= folder->permanent_flags;
 
 			/* weed out flag changes that we can't sync to the server */
@@ -658,11 +661,15 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 				camel_message_info_free(info);
 				continue;
 			} else {
+				gw_info->info.flags &= ~CAMEL_MESSAGE_FOLDER_FLAGGED;
 				gw_info->server_flags = gw_info->info.flags;
 				const char *uid = camel_message_info_uid (info);
-				if (diff.changed & CAMEL_MESSAGE_SEEN)
-					read_items = g_list_append (read_items, (char *)uid);
-				if (diff.changed & CAMEL_MESSAGE_DELETED) {
+				if (diff.bits & CAMEL_MESSAGE_DELETED) {
+
+					/* In case a new message is READ and then deleted immediately */
+					if (diff.bits & CAMEL_MESSAGE_SEEN)
+						deleted_read_items = g_list_prepend (deleted_read_items, (char *) uid);
+
 					if (deleted_items)
 						deleted_items = g_list_prepend (deleted_items, (char *)camel_message_info_uid (info));
 					else {
@@ -673,6 +680,25 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 					if (g_list_length (deleted_items) == GROUPWISE_BULK_DELETE_LIMIT ) {
 						CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+
+						/* 
+							Sync up the READ changes before deleting the message. 
+							Note that if a message is marked as unread and then deleted,
+							Evo doesnt not take care of it, as I find that scenario to be impractical.
+						*/
+
+						if (deleted_read_items) {
+							
+							/* FIXME: As in many places, we need to handle the return value
+							and do some error handling. But, we do not have all error codes also
+							and errors are not returned always either */
+
+							status = e_gw_connection_mark_read (cnc, deleted_read_items);
+							g_list_free (deleted_read_items);
+							deleted_read_items = NULL;
+						}
+
+						/* And now delete the messages */
 						status = e_gw_connection_remove_items (cnc, container_id, deleted_items);
 						CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
 						if (status == E_GW_CONNECTION_STATUS_OK) {
@@ -689,6 +715,10 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 							}
 						}
 					}
+				} else if (diff.bits & CAMEL_MESSAGE_SEEN) {
+					read_items = g_list_prepend (read_items, (char *)uid);
+				} else if (unset_flags.bits & CAMEL_MESSAGE_SEEN) {
+					unread_items = g_list_prepend (unread_items, (char *)uid);
 				}
 			}
 		}
@@ -721,10 +751,18 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 		g_list_free (deleted_head);
 	}
 
-	if (read_items && g_list_length (read_items)) {
+	if (read_items) {
 		CAMEL_SERVICE_LOCK (gw_store, connect_lock);
 		e_gw_connection_mark_read (cnc, read_items);
 		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		g_list_free (read_items);
+	}
+
+	if (unread_items) {
+		CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+		e_gw_connection_mark_unread (cnc, unread_items);
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		g_list_free (unread_items);
 	}
 
 	if (expunge) {
@@ -1149,6 +1187,8 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 	GList *item_list = list;
 	int total_items = g_list_length (item_list), i=0;
 
+	gboolean is_proxy = folder->parent_store->flags & CAMEL_STORE_WRITE;
+
 	CAMEL_SERVICE_ASSERT_LOCKED (gw_store, connect_lock);
 
 	changes = camel_folder_change_info_new ();
@@ -1177,6 +1217,8 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 		CamelStream *cache_stream, *t_cache_stream;
 		CamelMimeMessage *mail_msg = NULL;
 		gboolean is_sent_folder = FALSE;
+		const char *recurrence_key = NULL;
+		int rk;
 
 		exists = FALSE;
 
@@ -1188,7 +1230,7 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 
 		camel_operation_progress (NULL, (100*i)/total_items);
 
-		status = e_gw_connection_get_item (cnc, container_id, id, "peek default distribution recipient message attachments subject notification created recipientStatus status hasAttachment size", &item);
+		status = e_gw_connection_get_item (cnc, container_id, id, "peek default distribution recipient message attachments subject notification created recipientStatus status hasAttachment size recurrenceKey", &item);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
 			i++;
 			continue;
@@ -1214,9 +1256,15 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 			mi = (CamelGroupwiseMessageInfo *)camel_message_info_new (folder->summary); 
 			if (mi->info.content == NULL) {
 				mi->info.content = camel_folder_summary_content_info_new (folder->summary);
-				mi->info.content->type = camel_content_type_new ("multipart", "mixed");
+				mi->info.content->type = camel_content_type_new ("multipart", "mixed");	
 			}
 		}
+		
+		rk = e_gw_item_get_recurrence_key (item);
+		if (rk > 0) {
+			recurrence_key = g_strdup_printf("%d", rk); 
+			camel_message_info_set_user_tag ((CamelMessageInfo*)mi, "recurrence-key", recurrence_key);
+		} 
 
 		/*all items in the Junk Mail folder should have this flag set*/
 		if (is_junk)
@@ -1240,10 +1288,12 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 			mi->info.flags |= CAMEL_MESSAGE_FLAGGED;
 		}
 
-		mi->server_flags = mi->info.flags;
-		
 		if (e_gw_item_has_attachment (item))
 			mi->info.flags |= CAMEL_MESSAGE_ATTACHMENTS;
+                if (is_proxy)
+                        mi->info.flags |= CAMEL_MESSAGE_USER_NOT_DELETABLE;
+		
+		mi->server_flags = mi->info.flags;
 
 		org = e_gw_item_get_organizer (item); 
 		if (org) {
@@ -1379,6 +1429,8 @@ gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex)
 	gboolean is_junk = FALSE;
 	GList *item_list = list;
 
+	gboolean is_proxy = folder->parent_store->flags & CAMEL_STORE_WRITE;
+
 	/*Assert lock???*/
 	changes = camel_folder_change_info_new ();
 	container_id = g_strdup (camel_groupwise_store_container_id_lookup (gw_store, folder->full_name));
@@ -1400,6 +1452,8 @@ gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex)
 		const char *id;
 		GSList *recp_list = NULL;
 		status_flags = 0;
+		const char *recurrence_key = NULL;
+		int rk;
 
 		id = e_gw_item_get_id (item);
 
@@ -1420,6 +1474,12 @@ gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex)
 				mi->info.content->type = camel_content_type_new ("multipart", "mixed");
 			}
 		}
+		
+		rk = e_gw_item_get_recurrence_key (item);
+		if (rk > 0) {
+			recurrence_key = g_strdup_printf("%d", rk); 
+			camel_message_info_set_user_tag ((CamelMessageInfo*)mi, "recurrence-key", recurrence_key);
+		} 
 
 		/*all items in the Junk Mail folder should have this flag set*/
 		if (is_junk)
@@ -1445,6 +1505,11 @@ gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex)
 		if (e_gw_item_has_attachment (item))
 			mi->info.flags |= CAMEL_MESSAGE_ATTACHMENTS;
 
+		if (is_proxy) 
+			mi->info.flags |= CAMEL_MESSAGE_USER_NOT_DELETABLE;
+
+		mi->server_flags = mi->info.flags;
+
 		org = e_gw_item_get_organizer (item); 
 		if (org) {
 			GString *str;
@@ -1593,29 +1658,30 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 			EGwItemAttachment *attach = (EGwItemAttachment *)al->data;
 			if (!g_ascii_strcasecmp (attach->name, "Mime.822")) {
 				if (attach->size > MAX_ATTACHMENT_SIZE) {
-					long count = 0;
-					int i, t_len=0, offset=0, t_offset=0;
+					int len_iter = 0, t_len , offset = 0, t_offset = 0;
 					char *t_attach = NULL;
 					GString *gstr = g_string_new (NULL);
-
-					count = (attach->size)/(1024*1024);
-					count++;
+					
 					len = 0;
-					for (i = 0; i<count; i++) {
+					do {
 						status = e_gw_connection_get_attachment_base64 (cnc, 
 								attach->id, t_offset, MAX_ATTACHMENT_SIZE, 
 								(const char **)&t_attach, &t_len, &offset);
 						if (status == E_GW_CONNECTION_STATUS_OK) {
-							gstr = g_string_append (gstr, t_attach);
+							char *temp = NULL;
+	
+							temp = soup_base64_decode(t_attach, &len_iter);
+							gstr = g_string_append_len (gstr, temp, len_iter);
+							g_free (temp);
+							len += len_iter;
 							t_offset = offset;
 							g_free (t_attach);
 							t_attach = NULL;
-							t_len = 0;
 						}
-					}
-					body = soup_base64_decode (gstr->str, &len);
+					} while (offset);
+					body = gstr->str;
 					body_len = len;
-					g_string_free (gstr, TRUE);
+					g_string_free (gstr, FALSE);
 				} else {
 					status = e_gw_connection_get_attachment (cnc, 
 							attach->id, 0, -1, 
@@ -1729,28 +1795,31 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 				g_object_unref (temp_item);
 			} else {
 				if (attach->size > MAX_ATTACHMENT_SIZE) {
-					long count = 0;
-					int i, t_len=0, offset=0, t_offset=0;
+					int t_len=0, offset=0, t_offset=0;
 					char *t_attach = NULL;
 					GString *gstr = g_string_new (NULL);
 
-					count = (attach->size)/(1024*1024);
-					count++;
 					len = 0;
-					for (i = 0; i<count; i++) {
+					do {
 						status = e_gw_connection_get_attachment_base64 (cnc, 
 								attach->id, t_offset, MAX_ATTACHMENT_SIZE, 
 								(const char **)&t_attach, &t_len, &offset);
 						if (status == E_GW_CONNECTION_STATUS_OK) {
-							gstr = g_string_append (gstr, t_attach);
+							int len_iter = 0;
+							char *temp = NULL;
+	
+							temp = soup_base64_decode(t_attach, &len_iter);
+							gstr = g_string_append_len (gstr, temp, len_iter);
+							g_free (temp);
+							len += len_iter;
 							t_offset = offset;
 							g_free (t_attach);
 							t_attach = NULL;
 							t_len = 0;
 						}
-					}
-					attachment = soup_base64_decode (gstr->str, &len);
-					g_string_free (gstr, TRUE);
+					} while (t_offset);
+					attachment =  gstr->str;
+					g_string_free (gstr, FALSE);
 					is_base64_encoded = FALSE;
 				} else {
 					status = e_gw_connection_get_attachment (cnc, 
@@ -2026,13 +2095,64 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 		return;
 	}
 
-	cnc = cnc_lookup (priv);
 
+	cnc = cnc_lookup (priv);
 	index = 0;
 	while (index < uids->len) {
+		CamelMessageInfo *info = NULL;
+		CamelGroupwiseMessageInfo *gw_info = NULL;
+		flags_diff_t diff, unset_flags;
 		int count;
 		count = camel_folder_summary_count (destination->summary);
 
+		info = camel_folder_summary_uid (source->summary, uids->pdata[index]);
+		gw_info = (CamelGroupwiseMessageInfo *) info;
+		if (gw_info && (gw_info->info.flags & CAMEL_MESSAGE_FOLDER_FLAGGED)) {
+			do_flags_diff (&diff, gw_info->server_flags, gw_info->info.flags);
+			do_flags_diff (&unset_flags, gw_info->info.flags, gw_info->server_flags);
+			diff.changed &= source->permanent_flags;
+
+			/* sync the read changes */
+			if (diff.changed) {
+				const char *uid = camel_message_info_uid (info);
+				GList *wrapper = NULL;
+				gw_info->info.flags &= ~CAMEL_MESSAGE_FOLDER_FLAGGED;
+				gw_info->server_flags = gw_info->info.flags;
+
+				if (diff.bits & CAMEL_MESSAGE_SEEN) {
+					
+					/*
+					wrapper is a list wrapper bcos e_gw_connection_mark_read 
+					is designed for passing multiple uids. Also, there are is not much
+					need/use for a e_gw_connection_mark_ITEM_[un]read	
+					*/
+
+					wrapper = g_list_prepend (wrapper, (char *)uid);
+					CAMEL_SERVICE_LOCK (source->parent_store, connect_lock);
+					e_gw_connection_mark_read (cnc, wrapper);
+					CAMEL_SERVICE_UNLOCK (source->parent_store, connect_lock);
+					g_list_free (wrapper);
+					wrapper = NULL;
+				}
+
+				/* A User may mark a message as Unread and then immediately move it to
+				some other folder. The following piece of code take care of such scenario.
+				
+				However, Remember that When a mail is deleted after being marked as unread, 
+				I am not syncing the read-status. 
+				*/
+
+				if (unset_flags.bits & CAMEL_MESSAGE_SEEN) {
+					wrapper = g_list_prepend (wrapper, (char *)uid);
+					CAMEL_SERVICE_LOCK (source->parent_store, connect_lock);
+					e_gw_connection_mark_unread (cnc, wrapper);
+					CAMEL_SERVICE_UNLOCK (source->parent_store, connect_lock);
+					g_list_free (wrapper);
+					wrapper = NULL;
+				}
+			}
+		}
+
 		if (delete_originals) 
 			status = e_gw_connection_move_item (cnc, (const char *)uids->pdata[index], 
 					dest_container_id, source_container_id);
